@@ -6,19 +6,19 @@ MemBlocks supports any number of the following operations:
 
 * read: `d = mem[address]`
 * write: `mem[address] <<= d`
-* write with an enable: `mem[address] <<= MemBlock.EnabledWrite(d,enable=we)`
+* write with an enable: `mem[address] <<= MemBlock.EnabledWrite(d, enable=we)`
 
 Based on the number of reads and writes a memory will be inferred
 with the correct number of ports to support that
 """
 
-from __future__ import print_function, unicode_literals
 import collections
 
 from .pyrtlexceptions import PyrtlError
-from .core import working_block, LogicNet, _NameIndexer
+from .core import working_block, LogicNet, _NameIndexer, Block
 from .wire import WireVector, Const, next_tempvar_name
 from .corecircuits import as_wires
+from .helperfuncs import infer_val_and_bitwidth
 # ------------------------------------------------------------------------
 #
 #         ___        __   __          __        __   __
@@ -80,7 +80,6 @@ class _MemIndexed(WireVector):
     @property
     def name(self):
         return as_wires(self).name
-        # raise PyrtlError("MemIndexed is a temporary object and therefore doesn't have a name")
 
     @name.setter
     def name(self, n):
@@ -88,61 +87,115 @@ class _MemIndexed(WireVector):
 
 
 class MemBlock(object):
-    """ MemBlock is the object for specifying block memories.  It can be
-    indexed like an array for both reading and writing.  Writes under a conditional
-    are automatically converted to enabled writes.   For example, consider the following
-    examples where `addr`, `data`, and `we` are all WireVectors.
+    """MemBlock is the object for specifying block memories. It can be indexed like an
+    array for both reading and writing. Writes under a conditional are automatically
+    converted to enabled writes. Consider the following examples where ``addr``,
+    ``data``, and ``we`` are all WireVectors::
 
-    Usage::
+        data = memory[addr]  # create a read port
+        memory[addr] <<= data  # create a write port
+        mem[address] <<= MemBlock.EnabledWrite(data, enable=we)
 
-        data = memory[addr]  (infer read port)
-        memory[addr] <<= data  (infer write port)
-        mem[address] <<= MemBlock.EnabledWrite(data,enable=we)
+    When the address of a memory is assigned to using an
+    :class:`~.MemBlock.EnabledWrite` object, items will only be written to the memory
+    when the ``enable`` WireVector is set to high (1).
 
-    When the address of a memory is assigned to using a EnableWrite object
-    items will only be written to the memory when the enable WireVector is
-    set to high (1).
     """
     # FIXME: write ports assume that only one port is under control of the conditional
     EnabledWrite = collections.namedtuple('EnabledWrite', 'data, enable')
-    """ Allows for an enable bit for each write port, where data (the first field in
-        the tuple) is the normal data address, and enable (the second field) is a one
-        bit signal specifying that the write should happen (i.e. active high)."""
+    """Generates logic to conditionally enable a write port.
 
-    def __init__(self, bitwidth, addrwidth, name='', max_read_ports=2, max_write_ports=1,
-                 asynchronous=False, block=None):
-        """ Create a PyRTL read-write memory.
+    ``data`` (the first field in the tuple) is the data to write, and ``enable`` (the
+    second field) is a one bit signal specifying whether the write should happen (i.e.
+    active high).
 
-        :param int bitwidth: Defines the bitwidth of each element in the memory
-        :param int addrwidth: The number of bits used to address an element of the
-         memory. This also defines the size of the memory
-        :param str name: The identifier for the memory
-        :param max_read_ports: limits the number of read ports each
-            block can create; passing `None` indicates there is no limit
-        :param max_write_ports: limits the number of write ports each
-            block can create; passing `None` indicates there is no limit
-        :param bool asynchronous: If false make sure that memory reads are only done
-            using values straight from a register. (aka make sure that the
-            read is synchronous)
-        :param basestring name: Name of the memory. Defaults to an autogenerated
-            name
-        :param block: The block to add it to, defaults to the working block
+    """
 
-        It is best practice to make sure your block memory/fifos read/write
-        operations start on a clock edge if you want them to synthesize into efficient hardware.
-        MemBlocks will enforce this by making sure that
-        you only address them with a register or input, unless you explicitly declare
-        the memory as asynchronous with `asynchronous=True` flag.  Note that asynchronous mems
-        are, while sometimes very convenient and tempting, rarely a good idea.
-        They can't be mapped to block rams in FPGAs and will be converted to registers by most
-        design tools even though PyRTL can handle them with no problem.  For any memory beyond
-        a few hundred entries it is not a realistic option.
+    def __init__(self, bitwidth: int, addrwidth: int, name: str = '',
+                 max_read_ports: int = 2, max_write_ports: int = 1,
+                 asynchronous: bool = False, block: Block = None):
+        """Create a PyRTL read-write memory.
+
+        :param bitwidth: The bitwidth of each element in the memory.
+        :param addrwidth: The number of bits used to address an element in the memory.
+            The memory can store ``2 ** addrwidth`` elements.
+        :param name: Name of the memory. Defaults to an autogenerated name.
+        :param max_read_ports: limits the number of read ports each block can create;
+            passing ``None`` indicates there is no limit.
+        :param max_write_ports: limits the number of write ports each block can create;
+            passing ``None`` indicates there is no limit.
+        :param asynchronous: If ``False``, ensure that all memory inputs are registers,
+            inputs, or constants. See the note about asynchronous memories below.
+        :param block: The block to add the MemBlock to, defaults to the working block.
+
+        ---------------------
+        Asynchronous Memories
+        ---------------------
+        It is best practice to have memory operations start on a rising clock edge if
+        you want them to synthesize into efficient hardware. MemBlocks will enforce this
+        by checking that all their inputs are ready at each rising clock edge. This
+        implies that all MemBlock inputs - the address to read/write, the data to write,
+        and the write-enable bit - must be registers, inputs, or constants, unless you
+        explicitly declare the memory as ``asynchronous=True``.
+
+        Asynchronous memories can be convenient and tempting, but they are rarely a good
+        idea. They can't be mapped to block RAMs in FPGAs and will be converted to
+        registers by most design tools. They are not a realistic option for memories
+        with more than a few hundred elements.
+
+        --------------------
+        Read and Write Ports
+        --------------------
+        Each read or write to the memory will create a new `port` (either a read port or
+        write port respectively). By default memories are limited to 2 read ports and 1
+        write port, to keep designs efficient by default, but those values can be
+        changed. Note that memories with many ports may not map to physical memories
+        such as block RAMs or existing memory hardware macros.
+
+        --------------
+        Default Values
+        --------------
+        In PyRTL simulations, all MemBlocks are zero-initialized by default. Initial
+        data can be specified for each MemBlock in :meth:`.Simulation.__init__`'s
+        ``memory_value_map``.
+
+        ---------------------------
+        Simultaneous Read and Write
+        ---------------------------
+        In PyRTL simulations, if the same address is read and written in the same cycle,
+        the read will return the `last` value stored in the MemBlock, not the newly
+        written value. Example::
+
+            mem = pyrtl.MemBlock(addrwidth=1, bitwidth=1)
+            mem[0] <<= 1
+            read_data = pyrtl.Output(name="read_data", bitwidth=1)
+            read_data <<= mem[0]
+
+            # In the first cycle, read_data will be the default MemBlock data value (0),
+            # not the newly written value (1).
+            sim = pyrtl.Simulation()
+            sim.step()
+            print("Cycle 0 read_data", sim.inspect("read_data"))
+
+            # In the second cycle, read_data will be the newly written value (1).
+            sim.step()
+            print("Cycle 1 read_data", sim.inspect("read_data"))
+
+        -----------------------------
+        Mapping MemBlocks to Hardware
+        -----------------------------
+        Synchronous MemBlocks can generally be mapped to FPGA block RAMs and similar
+        hardware, but there are many pitfalls:
+
+        #. ``asynchronous=False`` is generally necessary, but may not be sufficient, for
+           mapping a design to FPGA block RAMs. Block RAMs may have additional timing
+           constraints, like requiring register outputs for each block RAM.
+           ``asynchronous=False`` only requires register inputs.
+        #. Block RAMs may offer more or less read and write ports than MemBlock's
+           defaults.
+        #. Block RAMs may not zero-initialize by default.
+        #. Block RAMs may implement simultaneous reads and writes in different ways.
 
-        Each read or write to the memory will create a new `port` (either a read port or write
-        port respectively).  By default memories are limited to 2-read and 1-write port, but
-        to keep designs efficient by default, but those values can be set as options.  Note
-        that memories with high numbers of ports may not be possible to map to physical memories
-        such as block rams or existing memory hardware macros.
         """
         self.max_read_ports = max_read_ports
         self.num_read_ports = 0
@@ -170,15 +223,15 @@ class MemBlock(object):
     def read_ports(self):
         raise PyrtlError('read_ports now called num_read_ports for clarity')
 
-    def __getitem__(self, item):
-        """ Builds circuitry to retrieve an item from the memory """
+    def __getitem__(self, item) -> WireVector:
+        """Create a read port to load items from the MemBlock."""
         item = as_wires(item, bitwidth=self.addrwidth, truncating=False)
         if len(item) > self.addrwidth:
             raise PyrtlError('memory index bitwidth > addrwidth')
         return _MemIndexed(mem=self, index=item)
 
     def __setitem__(self, item, assignment):
-        """ Builds circuitry to set an item in the memory """
+        """Create a write port to store items to the MemBlock."""
         if isinstance(assignment, _MemAssignment):
             self._assignment(item, assignment.rhs, is_conditional=assignment.is_conditional)
         else:
@@ -206,10 +259,10 @@ class MemBlock(object):
     def _assignment(self, item, val, is_conditional):
         from .conditional import _build
 
-        item = as_wires(item, bitwidth=self.addrwidth, truncating=False)
-        if len(item) > self.addrwidth:
-            raise PyrtlError('error, the wire indexing the memory bitwidth > addrwidth')
-        addr = item
+        # Even though as_wires is already called on item already in the __getitem__ method,
+        # we need to call it again here because __setitem__ passes the original item
+        # to _assignment.
+        addr = as_wires(item, bitwidth=self.addrwidth, truncating=False)
 
         if isinstance(val, MemBlock.EnabledWrite):
             data, enable = val.data, val.enable
@@ -255,36 +308,48 @@ class MemBlock(object):
 
 
 class RomBlock(MemBlock):
-    """ PyRTL Read Only Memory.
+    """PyRTL Read Only Memory.
+
+    RomBlocks are the read only memory block for PyRTL. They support the same read
+    interface as :class:`MemBlock`, but they cannot be written to (i.e. there are no
+    write ports). The ROM's contents are specified when the ROM is constructed.
 
-    RomBlocks are the read only memory block for PyRTL.  They support the same read interface
-    and normal memories, but they are cannot be written to (i.e. there are no write ports).
-    The ROM must be initialized with some values and construction through the use of the
-    `romdata` which is the memory for the system.
     """
-    def __init__(self, bitwidth, addrwidth, romdata, name='', max_read_ports=2,
-                 build_new_roms=False, asynchronous=False, pad_with_zeros=False, block=None):
-        """Create a Python Read Only Memory.
+    def __init__(self, bitwidth: int, addrwidth: int, romdata, name: str = '',
+                 max_read_ports: int = 2, build_new_roms: bool = False,
+                 asynchronous: bool = False, pad_with_zeros: bool = False,
+                 block: Block = None):
+        """Create a PyRTL Read Only Memory.
 
-        :param int bitwidth: The bitwidth of each item stored in the ROM
-        :param int addrwidth: The bitwidth of the address bus (determines number of addresses)
-        :param romdata: This can either be a function or an array (iterable) that maps
-            an address as an input to a result as an output
-        :param str name: The identifier for the memory
+        :param bitwidth: The bitwidth of each element in the ROM.
+        :param addrwidth: The number of bits used to address an element in the ROM.
+            The ROM can store ``2 ** addrwidth`` elements.
+        :param romdata: Specifies the data stored in the ROM. This can either be a
+            function or an array (iterable) that maps from address to data. Example::
+
+                # Create a 4-element ROM, where:
+                #   rom[0] == 4
+                #   rom[1] == 5
+                #   rom[2] == 6
+                #   rom[3] == 7
+                rom = RomBlock(bitwidth=3, addrwidth=2, romdata=[4, 5, 6, 7])
+        :param name: The identifier for the memory.
         :param max_read_ports: limits the number of read ports each block can create;
-            passing `None` indicates there is no limit
-        :param bool build_new_roms: indicates whether to create and pass new RomBlocks during
-            `__getitem__` to avoid exceeding `max_read_ports`
-        :param bool asynchronous: If false make sure that memory reads are only done
-            using values straight from a register. (aka make sure that reads
-            are synchronous)
-        :param bool pad_with_zeros: If true, extend any missing romdata with zeros out until the
-            size of the romblock so that any access to the rom is well defined.  Otherwise, the
-            simulation should throw an error on access of unintialized data.  If you are generating
-            verilog from the rom, you will need to specify a value for every address (in which case
-            setting this to True will help), however for testing and simulation it useful to know if
-            you are off the end of explicitly specified values (which is why it is False by default)
-        :param block: The block to add to, defaults to the working block
+            passing ``None`` indicates there is no limit.
+        :param build_new_roms: indicates whether :meth:`RomBlock.__getitem__` should
+            create copies of the RomBlock to avoid exceeding ``max_read_ports``.
+        :param asynchronous: If ``False``, ensure that all RomBlock inputs are
+            registers, inputs, or constants. See the notes about asynchronous memories
+            in :meth:`MemBlock.__init__`.
+        :param pad_with_zeros: If ``True``, fill any missing ``romdata`` with zeros so
+            all accesses to the ROM are well defined. Otherwise, the simulation will
+            throw an error when accessing unintialized data. If you are generating
+            Verilog, you will need to specify a value for every address (in which case
+            setting this to ``True`` will help), however for testing and simulation it
+            useful to know if you are accessing an unspecified value (which is why it is
+            ``False`` by default).
+        :param block: The block to add to, defaults to the working block.
+
         """
 
         super(RomBlock, self).__init__(bitwidth=bitwidth, addrwidth=addrwidth, name=name,
@@ -295,7 +360,13 @@ class RomBlock(MemBlock):
         self.current_copy = self
         self.pad_with_zeros = pad_with_zeros
 
-    def __getitem__(self, item):
+    def __getitem__(self, item) -> WireVector:
+        """Create a read port to load items from the RomBlock.
+
+        If ``build_new_roms`` was specified, create a new copy of the RomBlock if the
+        number of read ports exceeds ``max_read_ports``.
+
+        """
         import numbers
         if isinstance(item, numbers.Number):
             raise PyrtlError("There is no point in indexing into a RomBlock with an int. "
@@ -306,7 +377,13 @@ class RomBlock(MemBlock):
     def __setitem__(self, item, assignment):
         raise PyrtlError('no writing to a read-only memory')
 
-    def _get_read_data(self, address):
+    def _get_read_data(self, address: int):
+        """_get_read_data is called by the simulator to fetch RomBlock data.
+
+        :param address: address is a dynamic run-time value (an integer), *not* a
+            WireVector.
+
+        """
         import types
         try:
             if address < 0 or address > 2**self.addrwidth - 1:
@@ -326,7 +403,7 @@ class RomBlock(MemBlock):
                     value = 0
                 else:
                     raise PyrtlError(
-                        "RomBlock key is invalid, "
+                        f"RomBlock key {address} is invalid, "
                         "consider using pad_with_zeros=True for defaults"
                     )
             except IndexError:
@@ -334,15 +411,14 @@ class RomBlock(MemBlock):
                     value = 0
                 else:
                     raise PyrtlError(
-                        "RomBlock index is invalid, "
+                        f"RomBlock index {address} is invalid, "
                         "consider using pad_with_zeros=True for defaults"
                     )
             except Exception:
                 raise PyrtlError("invalid type for RomBlock data object")
 
         try:
-            if value < 0 or value >= 2**self.bitwidth:
-                raise PyrtlError("invalid value for RomBlock data")
+            value = infer_val_and_bitwidth(value, bitwidth=self.bitwidth).value
         except TypeError:
             raise PyrtlError("Value: {} from rom {} has an invalid type"
                              .format(value, self))
